%% Using saminfo to get how many total reads you have
function [Freads, Rreads, Total_reads] = SAMreader_TA (SAM_file, number_of_reads_blocked, chr_identifier, TAsites)

h=saminfo(SAM_file,'numofreads',true);
iterations=h.NumReads/number_of_reads_blocked; %calculates how many blocks we will go through--will be rounded down
readblock_start=1;
readblock_end=readblock_start + number_of_reads_blocked - 1;

Freads=zeros(length(TAsites),1);
Rreads=zeros(length(TAsites),1);

%% Section 1. For the first block

SAMfile=samread(SAM_file,'BlockRead',[readblock_start readblock_end],'tags',false); %imports reads from SAM_file
SAMfile2=struct2cell(SAMfile); % change the structure format of the data into a cell array
SAMfile2=SAMfile2'; % transpose the rows and columns, so every row is a read

Fmap_pos=[]; %creates vectors of data for later addition
Rmap_pos=[];
Rmap_seq={};

chr_ID=chr_identifier; %Search for reads that match the chromsome you want: 1 means the read matches 'chrI;; 0 means NOT chrI %this is INPUT ARGUMENT
for x=1:length(SAMfile2); %iterates through every row
    if SAMfile2{x,3} == chr_ID; % only does the loop if the read corresponds to the right chromosome indicated above
        if SAMfile2{x,2} == 0 % if the FLAG is a '0'
            Fmap_pos=[Fmap_pos;SAMfile2{x,4}]; % record the mapping position in the Fmap_pos list
        else
            if SAMfile2{x,2} == 16 % if the FLAG is '16'
                Rmap_pos=[Rmap_pos;SAMfile2{x,4}]; % record the mapping position in the Rmap_pos reads list
                Rmap_seq=[Rmap_seq;SAMfile2{x,10}]; % record the sequence of the read in the Rmap_pos list
            end
        end
    end
end

% we want to find the number of reads for each unique nt position
F_unique=unique(Fmap_pos); %give unique nt pos
F_tally=zeros(length(F_unique),1);%this is a vector tallying the for reads at each unique nt
for x=1:length(F_unique);
    F_tally(x,1)=sum(Fmap_pos==F_unique(x));
end

% Have to alter the R mapping positions: this is because bowtie maps the
% leftmost nt, which doesn't correspond to the insertion site on the R
% strand.  Will have to add the length of the sequence and -2 (b/c of
% Bowtie default mapping paramters).

R_calc=zeros(length(Rmap_pos),1);
for x=1:length(Rmap_pos);
     R_calc(x,1)=(Rmap_pos(x,1) + (length(Rmap_seq{x}))-2);
end

R_unique=unique(R_calc);
R_tally=zeros(length(R_unique),1);

for x=1:length(R_unique);
    R_tally(x,1)=sum(R_calc==R_unique(x));
end

%% Section 2. This section maps the F and R tallied reads to the appropriate TA site in the genome
for x=1:length(F_unique);
    if any(F_unique(x)==TAsites) == 1;
        s = (F_unique(x)==TAsites);
        t = find(s,1);
        Freads(t,1)=[F_tally(x)+Freads(t,1)];
    end
end

%
for x=1:length(R_unique);
    if any(R_unique(x)==TAsites) == 1;
        s = (R_unique(x)==TAsites);
        t = find(s,1);
        Rreads(t,1)=[R_tally(x)+Rreads(t,1)];
    end
end

%% Section 3. For the remaining blocks of SAM reads
% Load rows of a SAM file--performs rest of FXN, then loops back and and upload another batch of reads.  500000 reads took about 5 mins on a 4GB machine to load
for i=1:iterations; %technically, this starts at the second block of reads and goes until 1 block more than the last whole interval (i.e, past the end of the SAM file to capture all remaining reads)
    SAMfile=samread(SAM_file,'BlockRead',[readblock_start readblock_end],'tags',false);
    readblock_start=readblock_end+1;
    readblock_end=readblock_start + number_of_reads_blocked-1;
    
    SAMfile2=struct2cell(SAMfile); % change the structure format of the data into a cell array
    SAMfile2=SAMfile2'; % transpose the rows and columns, so every row is a read
    
    Fmap_pos=[]; %creates vectors of data for later addition
    Rmap_pos=[];
    Rmap_seq={};
     
    chr_ID=chr_identifier; %Search for reads that match the chromsome you want: 1 means the read matches 'chrI;; 0 means NOT chrI %this is INPUT ARGUMENT
    for x=1:length(SAMfile2); %iterates through every row
        if SAMfile2{x,3} == chr_ID; % only does the loop if the read corresponds to the right chromosome indicated above
            if SAMfile2{x,2} == 0 % if the FLAG is a '0'
                Fmap_pos=[Fmap_pos;SAMfile2{x,4}]; % record the mapping position in the Fmap_pos list
            else
                if SAMfile2{x,2} == 16 % if the FLAG is '16'
                    Rmap_pos=[Rmap_pos;SAMfile2{x,4}]; % record the mapping position in the Rmap_pos reads list
                    Rmap_seq=[Rmap_seq;SAMfile2{x,10}]; % record the sequence of the read in the Rmap_pos list
                end
            end
        end
    end

    % we want to find the number of reads for each unique nt position
    F_unique=unique(Fmap_pos); %give unique nt pos
    F_tally=zeros(length(F_unique),1);%this is a vector tallying the for reads at each unique nt
    for x=1:length(F_unique);
        F_tally(x,1)=sum(Fmap_pos==F_unique(x));
    end

    % Have to alter the R mapping positions: this is because bowtie maps the
    % leftmost nt, which doesn't correspond to the insertion site on the R
    % strand.  Will have to add the length of the sequence and -2 (b/c of
    % Bowtie default mapping paramters).

    R_calc=zeros(length(Rmap_pos),1);
    for x=1:length(Rmap_pos);
         R_calc(x,1)=(Rmap_pos(x,1) + (length(Rmap_seq{x}))-2);
    end

    R_unique=unique(R_calc);
    R_tally=zeros(length(R_unique),1);

    for x=1:length(R_unique);
        R_tally(x,1)=sum(R_calc==R_unique(x));
    end

    % Looks for each Unique site in TAsites--is this read a legitamate
    % insertion at the TA site.
    for x=1:length(F_unique);
        if any(F_unique(x)==TAsites) == 1;
            s = (F_unique(x)==TAsites);
            t = find(s,1);
            Freads(t,1)=[F_tally(x)+Freads(t,1)];
        end
    end

    %
    for x=1:length(R_unique);
        if any(R_unique(x)==TAsites) == 1;
            s = (R_unique(x)==TAsites);
            t = find(s,1);
            Rreads(t,1)=[R_tally(x)+Rreads(t,1)];
        end
    end
end

%% Section 4.  Combines the F and R reads into the Control or Experiment dataset to feed into ARTIST
Total_reads=Freads+Rreads; %This will be either the Control or Experiment data for ARTIST!





