%% Using saminfo to get how many total reads you have
function [Tn5_NT, All_Tn5_sum, Tn5_annotations, F_Tn5_sum, R_Tn5_sum] = SAMreader_Tn5(Chr1_ends, Chr1_starts, Chr1_names,SAM_file, number_of_reads_blocked, window_size, chr_identifier, Genome_Pos)

h=saminfo(SAM_file,'numofreads',true);
iterations=h.NumReads/number_of_reads_blocked; %calculates how many blocks we will go through--will be rounded down
readblock_start=1;
readblock_end=readblock_start + number_of_reads_blocked - 1;
numwindows=max(Genome_Pos)/window_size;
fixwind=floor(numwindows); %rounds the window sizes down to nearest whole number;

%% Section 1. For the first block

SAMfile=samread(SAM_file,'BlockRead',[readblock_start readblock_end],'tags',false); %imports reads from SAM_file
SAMfile2=struct2cell(SAMfile); % change the structure format of the data into a cell array
SAMfile2=SAMfile2'; % transpose the rows and columns, so every row is a read

Fmap_pos=[]; %creates vectors of data for later addition
Rmap_pos=[];
Rmap_seq={};

chr_ID=chr_identifier; %Search for reads that match the chromsome you want: 1 means the read matches 'chrI;; 0 means NOT chrI %this is INPUT ARGUMENT
for x=1:length(SAMfile2); %iterates through every row
    if SAMfile2{x,3} == chr_ID; % only does the loop if the read corresponds to the right chromosome indicated above
        if SAMfile2{x,2} == 0 % if the FLAG is a '0'
            Fmap_pos=[Fmap_pos;SAMfile2{x,4}]; % record the mapping position in the Fmap_pos list
        else
            if SAMfile2{x,2} == 16 % if the FLAG is '16'
                Rmap_pos=[Rmap_pos;SAMfile2{x,4}]; % record the mapping position in the Rmap_pos reads list
                Rmap_seq=[Rmap_seq;SAMfile2{x,10}]; % record the sequence of the read in the Rmap_pos list
            end
        end
    end
end

% we want to find the number of reads for each unique nt position
F_unique=unique(Fmap_pos); %give unique nt pos
F_tally=zeros(length(F_unique),1);%this is a vector tallying the for reads at each unique nt

for x=1:length(F_unique);
    F_tally(x,1)=sum(Fmap_pos==F_unique(x));
end

% Have to alter the R mapping positions: this is because bowtie maps the
% leftmost nt, which doesn't correspond to the insertion site on the R
% strand.  Will have to add the length of the sequence and -2 (b/c of
% Bowtie default mapping paramters).

R_calc=zeros(length(Rmap_pos),1);
for x=1:length(Rmap_pos);
    R_calc(x,1)=(Rmap_pos(x,1) + (length(Rmap_seq{x}))-2);
end

R_unique=unique(R_calc);
R_tally=zeros(length(R_unique),1);

for x=1:length(R_unique);
    R_tally(x,1)=sum(R_calc==R_unique(x));
end

%% Section 2. This section maps the F and R tallied reads to the appropriate NT (Tn5) site in the genome
Tn5_NT=zeros(fixwind+1,1); %defining the middle nt, functionally the "TA" for that window

F_Tn5_sum=zeros(fixwind+1,1);%this is the sum of the reads for the window

for x=1:fixwind; %iterate through the windows
    Tn5_NT(x)=(x*window_size)-round(window_size/2); %creates a list of windows and their midpoint positions
    for p=1:length(F_unique);
        if (F_unique(p)<=x*window_size & F_unique(p)>=x*window_size-window_size+1)==1; %if the F read is in the window
            F_Tn5_sum(x)=F_tally(p)+ F_Tn5_sum(x); %tally the reads for that window (NT)
        end
    end
end

if (Genome_Pos - (x * window_size)) > 0 % tally the forward reads in final window
    Tn5_NT(x+1)=Genome_Pos; %if there is leftover sequence after the last window, create a window with the last nt as the anchor point
    for p=1:length(F_unique);
        if(F_unique(p)<=Genome_Pos & F_unique(p)>=x*window_size+1)==1; %if the F read is in the window
            F_Tn5_sum(x+1)=F_tally(p)+ F_Tn5_sum(x+1);
        end
    end
end

%now tally the reverse
R_Tn5_sum=zeros(fixwind+1,1);%this is the sum of the reads for the window

for x=1:fixwind;
    for p=1:length(R_unique);
        if(R_unique(p)<=x*window_size & R_unique(p)>=x*window_size-window_size+1)==1;
            R_Tn5_sum(x)=R_tally(p)+ R_Tn5_sum(x);
        end
    end
end

if (Genome_Pos - (x * window_size)) > 0 % Tally Reverse reads in final window
    for p=1:length(R_unique);
        if(R_unique(p)<=Genome_Pos & R_unique(p)>=x*window_size+1)==1; %if the F read is in the window
            R_Tn5_sum(x+1)=R_tally(p)+ R_Tn5_sum(x+1);
        end
    end
end


%% Section 3. For the remaining blocks of SAM reads
% Load rows of a SAM file--performs rest of FXN, then loops back and and upload another batch of reads.  500000 reads took about 5 mins on a 4GB machine to load
for i=1:iterations; %technically, this starts at the second block of reads and goes until 1 block more than the last whole interval (i.e, past the end of the SAM file to capture all remaining reads)
    SAMfile=samread(SAM_file,'BlockRead',[readblock_start readblock_end],'tags',false);
    readblock_start=readblock_end+1;
    readblock_end=readblock_start + number_of_reads_blocked-1;
    
    SAMfile2=struct2cell(SAMfile); % change the structure format of the data into a cell array
    SAMfile2=SAMfile2'; % transpose the rows and columns, so every row is a read
    
    Fmap_pos=[]; %creates vectors of data for later addition
    Rmap_pos=[];
    Rmap_seq={};
    
    chr_ID=chr_identifier; %Search for reads that match the chromsome you want: 1 means the read matches 'chrI;; 0 means NOT chrI %this is INPUT ARGUMENT
    for x=1:length(SAMfile2); %iterates through every row
        if SAMfile2{x,3} == chr_ID; % only does the loop if the read corresponds to the right chromosome indicated above
            if SAMfile2{x,2} == 0 % if the FLAG is a '0'
                Fmap_pos=[Fmap_pos;SAMfile2{x,4}]; % record the mapping position in the Fmap_pos list
            else
                if SAMfile2{x,2} == 16 % if the FLAG is '16'
                    Rmap_pos=[Rmap_pos;SAMfile2{x,4}]; % record the mapping position in the Rmap_pos reads list
                    Rmap_seq=[Rmap_seq;SAMfile2{x,10}]; % record the sequence of the read in the Rmap_pos list
                end
            end
        end
    end
    
    % we want to find the number of reads for each unique nt position
    F_unique=unique(Fmap_pos); %give unique nt pos
    F_tally=zeros(length(F_unique),1);%this is a vector tallying the for reads at each unique nt
    
    for x=1:length(F_unique);
        F_tally(x,1)=sum(Fmap_pos==F_unique(x));
    end
    
    % Have to alter the R mapping positions: this is because bowtie maps the
    % leftmost nt, which doesn't correspond to the insertion site on the R
    % strand.  Will have to add the length of the sequence and -2 (b/c of
    % Bowtie default mapping paramters).
    
    R_calc=zeros(length(Rmap_pos),1);
    for x=1:length(Rmap_pos);
        R_calc(x,1)=(Rmap_pos(x,1) + (length(Rmap_seq{x}))-2);
    end
    
    R_unique=unique(R_calc);
    R_tally=zeros(length(R_unique),1);
    
    for x=1:length(R_unique);
        R_tally(x,1)=sum(R_calc==R_unique(x));
    end
    
    % Looks for each Unique site in TAsites--is this read a legitamate
    % insertion at the TA site.
    % this is for the last block i think?
    
    for x=1:fixwind;
        for p=1:length(F_unique);
            if(F_unique(p)<=x*window_size & F_unique(p)>=x*window_size-window_size+1)==1;
                F_Tn5_sum(x)=F_tally(p)+ F_Tn5_sum(x);
            end
        end
    end
    
    if (Genome_Pos - (x * window_size)) > 0 % tally the forward reads in final window
        for p=1:length(F_unique);
            if(F_unique(p)<=Genome_Pos & F_unique(p)>=x*window_size+1)==1; %if the F read is in the window
                F_Tn5_sum(x+1)=F_tally(p)+ F_Tn5_sum(x+1);
            end
        end
    end
    
    %now tally the reverse
    
    for x=1:fixwind;
        for p=1:length(R_unique);
            if(R_unique(p)<=x*window_size & R_unique(p)>=x*window_size-window_size+1)==1;
                R_Tn5_sum(x)=R_tally(p)+ R_Tn5_sum(x);
            end
        end
    end
    
    if (Genome_Pos - (x * window_size)) > 0 % Tally Reverse reads in final window
        for p=1:length(R_unique);
            if(R_unique(p)<=Genome_Pos & R_unique(p)>=x*window_size+1)==1; %if the F read is in the window
                R_Tn5_sum(x+1)=R_tally(p)+ R_Tn5_sum(x+1);
            end
        end
    end
end

% Now add the F and R reads (per window) together
All_Tn5_sum=zeros(fixwind+1,1);
All_Tn5_sum= F_Tn5_sum + R_Tn5_sum;

if (Genome_Pos-(x * window_size)) == 0 % if we do not have a real last window (e.g, chromosome is multiple of window_size)
    Tn5_NT = Tn5_NT(1:end-1);
    R_Tn5_sum=R_Tn5_sum(1:end-1);
    F_Tn5_sum=F_Tn5_sum(1:end-1);
    All_Tn5_sum=All_Tn5_sum(1:end-1);
end

%now find annotations for each NT (for each 'window')
Tn5_annotations={};
for x=1:length(Tn5_NT);
    for q=1:length(Chr1_ends);
        if (Tn5_NT(x)<=Chr1_ends(q) & Tn5_NT(x)>=Chr1_starts(q)==1);
            Tn5_annotations(x)= Chr1_names(q);
            break
        end
    end
end

Tn5_annotations=Tn5_annotations';




