#Control and R to run lines
#?function to get help file


## This analysis needs the insertion index score. 

#-----SET DIRECTORY-------

#set the right directory and check you're there

rm(list=ls())

getwd()

setwd("~/Dropbox/Climb_TraDIS/Evolution_analysis_24_04_19/Evolution_24_04_19/Download_Evolution//ITL_comb")

getwd()

#dev.off() #opens a graphic device - need this in Rstudio



#------SET CONDITION------

condition = "test"



#------READ IN DATA------

#read in the data and check it's there

info<-read.csv("insertion_indices.csv", header=FALSE)

II<-matrix(info[,2],ncol=1)

ls()






##------CHECK YOUR DATA---------

#Number of genes in the study
nG=dim(II)[1]
nG






#------PLOT YOUR DATA---------
## This line of code is the original code (Langridge et al. 2009)
#hist(II, breaks=0:2000/1000,xlim=c(0,0.14))
#png(filename="hist1.png",units="in",width=10,height=5,res=300)


## This is updated version to draw histogram with data-informed bin widths (Goodall et al. 2018)
hfull<-hist(II, breaks="FD",xlim=c(0,0.8))

#dev.off()





#------CREATE FIRST MODE DATA SET--------

#select the data to the left of the second peak (get this value from the above histogram)
I<-II<0.01
#plot a histogram of the above data
h<-hist(II[I],breaks="FD",plot=TRUE,xlim=c(0,0.8),xlab="Insertion Index",main=NULL)

## original code
#h<-hist(II[I],breaks=11:20/1000,plot=TRUE,xlim=c(0,0.14),ylim=c(0,80))



#locate the x-axis value where it first hits the minimum (i.e. where to split the modes)
m<-h$mids[which.min(h$density)]
m

#Call I1 all the data from this first mode (without the zero entries)
I1<-((II<m)&(II>0))
#I1

#png(filename="hist2.png",units="in",width=10,height=5,res=300)

histmode1<-hist(II[I1],breaks="FD",plot=TRUE,xlim=c(0,0.3))

#dev.off()






#-----CREATE SECOND MODE DATA SET------------

#select all data to the left of the tail and plot it
I<-II<1

h<-hist(II[I],breaks="FD",plot=TRUE,xlim=c(0,0.8))

#Call I2 all the data from the end of the first mode to the last place where there are at least 5 counts

I2<-((II>=m)&(II<h$mids[max(which(h$counts>5))]))

#png(filename="hist3.png",units="in",width=10,height=5,res=300)

histmode2<-hist(II[I2],breaks="FD",plot=TRUE,xlim=c(0,0.8))

#dev.off()




#----CALCULATE MODE FRACTIONS FOR SCALING LATER---------


#fraction of genes in first mode

f1<-(sum(I1)+sum(II==0))/nG

#fraction of genes in second mode

f2<-(sum(I2))/nG




#-----FIT THE MODES TO DISTRIBUTIONS--------

library(MASS)

d1<-fitdistr(II[I1],'exponential')

d2<-fitdistr(II[I2],'gamma')

d1
d2






#---PLOT PROBILITY MASS FUNCTION, PROBABILITY DISTRIBUTION FUNCTION AND FREQUENCIES ON ONE PLOT-----

totalcount<-sum(hfull$counts) #should be 4313 for BW25113
#rescale the frequency count with total count to generate probability mass functions
hfull$counts<-hfull$counts/totalcount
par(mar=c(5.1,4.1,4.1,4))
plot(hfull,ylab="Probability mass function",main=NULL,xlab="Insertion Index")


#scale fitted pdfs with fraction of genes in their portion and bin width to move from pmf to density
par(col="red")
lines(0:400/500,f1*(hfull$breaks[2]-hfull$breaks[1])*dgamma(0:400/500,1,d1$es[1]))
par(col="blue")
lines(0:400/500,f2*(hfull$breaks[2]-hfull$breaks[1])*dgamma(0:400/500,d2$es[1],d2$es[2]))



#plot frequencies with axis on RHS (they should overlay perfectly)
par(new=TRUE)
par(mar=c(5.1,4.1,4.1,4))
hfull$counts<-hfull$counts*totalcount
plot(hfull,xaxt="n",yaxt="n",xlab="",ylab="", main=NULL)
axis(4)
mtext("Frequency",side=4,line=3,col="black")

#dev.off()

#when the two distributions switch over effectively (where the blue and red lines intersect)

print(paste("Changepoint at",max(which(f1*dexp(1:1500/5000,d1$es)>f2*dgamma(1:1500/5000,d2$es[1],d2$es[2])))/5000))





#------CALCULATE LOG LIKELIHOODS---------

p_non_ess<-pgamma(II,d2$es[1],d2$es[2],lower.tail=TRUE)

p_ess<-pgamma(II,1,d1$es[1],lower.tail=TRUE)

loglike=-log2(dgamma(II,1,d1$es[1])+1E-50)+log2(dgamma(II,d2$es[1],d2$es[2])+1E-50)






#-----DETERMINE GENE ESSENTIALITIY-----

##12 times more likely
Essential<-(loglike<(-3.6))
NonEssential<-(loglike>(3.6))
Unclear<-(loglike>=(-3.6) & loglike<=(3.6))


##32 times more likely
#Essential<-(loglike<(-5))
#NonEssential<-(loglike>(5))
#Unclear<-(loglike>=(-5) & loglike<=(5))

## 4fold
#Essential<-(loglike<(-2))
#NonEssential<-(loglike>(2))
#Unclear<-(loglike>=(-2) & loglike<=(2))



#------SAVE THE DATA-------

info[,4]<-loglike
info[,5]<-Essential
info[,6]<-NonEssential
info[,7]<-Unclear

info[ , c(2,3)] <- info[ , c(3,2)]

colnames(info) <- c("GFF_gene_name","Gene_name", "Insertion_index_score", 'Log_likelihood', 'Essential', 'Non-essential', 'Unclear')

#setwd("Z/Dropbox/Climb_TraDIS/Evolution_analysis_24_04_19/Langbridge_et_al_Analysis")
write.table(info,file=paste(condition,"Day10_pH7_comb_results_FD.csv", sep=""),sep=",",row.names=FALSE,col.names=TRUE)






#---------CREATES A LIST OF ESSENTIAL AND NONESSENTIAL GENES---
gene_names<-matrix(info[,2],ncol=1)

a<-which(loglike<(-3.6))
essential_gene_list<-gene_names[a]
write.table(essential_gene_list,file=paste(condition, "Day10_pH7_comb_essential_genes_FD.csv", sep=""),sep=",",row.names=FALSE,col.names = FALSE)
b<-which(loglike>3.6)
nonessential_gene_list<-gene_names[b]
write.table(nonessential_gene_list,file=paste(condition, "Day10_pH7_comb_nonessential_genes_FD.csv", sep=""),sep=",",row.names=FALSE,col.names = FALSE)
c<-which((-3.6<loglike)&(loglike<(3.6)))
nonessential_gene_list<-gene_names[c]
write.table(nonessential_gene_list,file=paste(condition, "Day10_pH7_comb_unclear_genes_FD.csv", sep=""),sep=",",row.names=FALSE,col.names = FALSE)
Day




