
##############################################################################################
## LOADING IN FILES
##############################################################################################
## Loading in the annotations that you want. This can be generated through David website. 
gene_list <- read.csv("Annotations.csv", stringsAsFactors = FALSE)

## optional background gene list
#background_gene_list= read.csv("background_gene_list", stringsAsFactors = FALSE)


########################################################################################
################# Getting Entrez IDs from the Ecoli K12 data ##################################
########################################################################################

library("org.EcK12.eg.db")

gene_list <- c()# Imput a fector of any gene list requried. 
geneIDs<-select(org.EcK12.eg.db, gene_list, c("ENTREZID","GENENAME"),"SYMBOL")
org.EcK12.egALIAS2EG
#write.csv(geneIDs, "Entrez_ids_for_essential_genes.csv")
Mapped_Genes <- geneIDs[,2] ## genes to use in the next tab.
########################################################################################
## GO TERMS ENRICHMENT USING DAVID 
########################################################################################

library("RDAVIDWebService")

### Login to David 

david<-DAVIDWebService(email="mtm628@bham.ac.uk", url="https://david.ncifcrf.gov/webservice/services/DAVIDWebService.DAVIDWebServiceHttpSoap12Endpoint/") ## Note use your own licence


## Printing how many genes are within your analysis 
## If using a an alterantive background list. 
#print(paste("your geneome consits of", paste(nrow(annotations),"genes")))
#print(paste("in this analysis you have", paste(length(background_gene_list), "genes, in this analysis.")))


### Getting the annotation lists into David.. 

# Forground list  
FG <- addList(david, Mapped_Genes,idType = "ENTREZ_GENE_ID",listName = "isClass", listType = "Gene")

## Background List. This is important as this is the correct file to begin with. 
#BG <- addList(david, background_gene_list, idType = "ENTREZ_GENE_ID", listName = "background", listType = "Background")

#Making sure the list are loaded into david 
#FG
#BG
d#avid 

## Check the background list is the one you want. if not Setting the background list based on number order
#setCurrentBackgroundPosition(david,15)


## Call the relevant catogaries which are intrested in up 
## Get all the catorgary names for the analysis 
#getAllAnnotationCategoryNames(david)
## Getting information of a variety of catorgaries choose based on your option 
setAnnotationCategories(david, c("GOTERM_BP_3", "GOTERM_MF_3", "GOTERM_CC_3","GOTERM_BP_ALL", "GOTERM_MF_ALL", "GOTERM_CC_ALL","GOTERM_BP_1", "GOTERM_MF_1", "GOTERM_CC_1" ,"GOTERM_BP_2", "GOTERM_MF_2", "GOTERM_CC_2" ,"COG_ONTOLOGY","KEGG_PATHWAY","UP_KEYWORDS"))
## Getting the annotation chart

FuncAnnotChart <- getFunctionalAnnotationChart(david)
# Saving the annotation chart. 
getFunctionalAnnotationChartFile(david, "<unkown>func_ann_start.tsv")


## Seperating the different Functional parts of the annotation chart 

## COnverting FUnct annotation chart to a dataframe

BP <- FuncAnnotChart[FuncAnnotChart[,1] %in% "GOTERM_BP_2",]
MF <- FuncAnnotChart[FuncAnnotChart[,1] %in% "GOTERM_MF_2",]
CC <- FuncAnnotChart[FuncAnnotChart[,1] %in% "GOTERM_CC_2",]
COG <- FuncAnnotChart[FuncAnnotChart[,1] %in% "COG_ONTOLOGY",]
Kegg <- FuncAnnotChart[FuncAnnotChart[,1] %in% "KEGG_PATHWAY",]
Entrez <- FuncAnnotChart[FuncAnnotChart[,1] %in% "ENTREZ_GENE_SUMMARY",]
UP <- FuncAnnotChart[FuncAnnotChart[,1] %in% "UP_KEYWORDS" ,] ## UNIPROT KEYWORDS
# Making a chart which is needed the - log 10 

########################################################################################################

#################################### PLOTTING GRAPHS BASED ON 

########################################################################################################
## Plotting any of these graphs 

library("ggplot2")


## Function to plot a GO TERM Graph requires the data function above to get the data. 
## Data need to be two colomns labled Term i.e go terms, and another collum which contains pval which can be named anything 
## function will sort the data calculated the -log10(pval) and plot this value. so the larger the bar the more significant 

#file of interest- file which contains the data of interest 
FOI <- BP

## satistical test to choose pvalue
## Can either be  "Bonferroni"      "Benjamini"       "FDR"or not FDR  "PValue"
satistical_test <- "Benjamini"
data <- FOI[,colnames(BP)%in% c("Term",satistical_test)]

## Sorting by significance 

pval = 0.05

#####################################
## Func 1 : Plotting based on significant data and Pvalue Go data already sorted 
#####################################

Goterm_barplot <- function(condition_name,data,pval){
  
  # sorted by pval 
  
  sig_data <- data[data[,2] <= pval,]
  
  # changing the pval to -log10(pval)
  sig_data_log <- sig_data
  sig_data_log[,2] <- -log10(sig_data[,2])
  colnames(sig_data_log) <- c("Term", "-log10(pval)")
  sig_data_log <- sig_data_log[order(sig_data_log$`-log10(pval)`,decreasing = TRUE),]

ggplot(data=sig_data_log, aes(x=reorder(Term,`-log10(pval)`),y=`-log10(pval)`)) +
  geom_bar(stat="identity",fill="deepskyblue3")+
  xlab(condition_name)+
  theme_bw()+
  coord_flip()
}

############################################################################################
#### Plotting Terms on in a pie diagram
#############################################################
data <- UP

Percentage_graph <- function(data){
NO_genes <- mean(data$List.Total)
Table <- data[,c(2,3)]
Table$Percentage = (Table$Count/NO_genes)*100

ggplot(data=Table, aes(x=reorder(Term,`Percentage`),y=`Percentage`)) +
  geom_bar(stat="identity",fill="deepskyblue3")+
  ylim(0,100) +
  xlab(data[1,1])+
  theme_bw()+
  coord_flip()
}

pdf("MG1655_ITL_FUNCTION_graphs.pdf")
Percentage_graph(BP)
Percentage_graph(MF)
Percentage_graph(CC)
Percentage_graph(Kegg)
Percentage_graph(UP)
dev.off()


ggplot(data=Table, aes(x="",y=`Percentage`,fill=Term)) +
  geom_bar(stat="identity")+
  #xlab(condition_name)+
  coord_polar("y")+
  theme_bw()
}

################################### 
## 2 Plotting based on original GO term output So from original Go output Utlizes 1 
######################################

### FOI file of intrest, so file which has been split by Go ontology BP,MP,CC,or kegg or COG
FOI <- UP

### satistical test can be either "Bonferroni" ,"Benjamini" ,"FDR", "PValue"
## Condition_name is the name of the sample 

Plotting_Goterm <- function(FOI,satistical_test,pval,condition_name){
  satistical_test <- "Bonferroni"
  data <- FOI[,colnames(BP)%in% c("Term",satistical_test)]
  
  Goterm_barplot(condition_name,data,pval)
  
}

#test
FOI <- UP
satistical_test <- "Benjamini"
pval<- 0.05
condition_name <- "TEST"
Plotting_Goterm(FOI,satistical_test,pval,condition_name)

